/*
  * Unitex 
  *
  * Copyright (C) 2001-2008 Universit� de Marne-la-Vall�e <unitex@univ-mlv.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */

/* Created by Agata Savary (agata.savary@univ-tours.fr)
 * Last modification on September 15 2009
 */
//---------------------------------------------------------------------------

/********************************************************************************/
//// Debugging functionalities for inflection graphs                           ///
/********************************************************************************/

#ifndef MF_GraphDebugH
#define MF_GraphDebugH

#include "Unicode.h"


//Constant corresponding to running Multiflex in debug mode
#define GRAPH_DEBUG 1
//Constant corresponding to running Multiflex in non-debug mode
#define GRAPH_NO_DEBUG 0

//////////////////////////////////////////////////
// A graph path followed while generating a given form
// E.g. [7,<1,1,2,3,3,5,4>] means that the form
// can be generated by the following path containing 7 state and transition numbers:
// From state 1 tough label 1 to state 2, through label 3 to state 3
// and through label 5 to state 4
typedef struct {
  int len;    //Lenght of the path (i.e. number of state and transition numbers)
  int* path;  //Sequence of state and transition numbers
} gr_path_T;

/////////////////////////////////////////////////
// A set of graph paths allowing to generate the same form
// E.g. [2,<[7,<1,1,2,3,3,5,4>],[3,<1,4,5>]>]
// contains two paths of lenght 7 and 3 respectively
typedef struct {
  int no_paths;     //Numbers of paths
  gr_path_T* paths; //Set of paths
} graph_paths_T; 


////////////////////////////////////////////
// Initialize the list 'paths' with null values
// We suppose that 'paths' has its space allocated
void gp_init_graph_paths(graph_paths_T* paths);

////////////////////////////////////////////
// Add an empty path to the set of paths 
// We suppose that 'path' has its space allocated
void gp_add_empty_path(graph_paths_T* paths);

////////////////////////////////////////////
// Add the set paths 'new_path' to the set of paths 'paths' without repetitions
// We suppose that 'paths' has its space allocated
// Return 0 on success, 1 otherwise.   
int gp_merge_paths(graph_paths_T* paths, graph_paths_T* new_paths);

////////////////////////////////////////////
// Add the transition or state number 'ts' at the beginning of each path in 'p'
// Return 1 in case of errors, 0 otherwise
int gp_add_transition_or_state(graph_paths_T* paths, int ts);

////////////////////////////////////////////
// Duplicate the paths 'p2' into 'p1'
// 'p1' is allocated but empty
// Returns 0 on success, 1 on error
int gp_duplicate_graph_paths(graph_paths_T* p1,graph_paths_T* p2);

////////////////////////////////////////////
// Gets a string from a set of graph paths. The paths are separated by a ';'.
// The return string is allocated in the function. The liberation has to take place  
// in the calling function.                                                         
unichar* gp_get_str_graph_paths(graph_paths_T* paths);

////////////////////////////////////////////
// Prints a set of paths
void gp_print_graph_paths(graph_paths_T* paths);

////////////////////////////////////////////
// Deletes a set of paths
// The main structure is not liberated
void gp_delete_graph_paths(graph_paths_T* paths);

#endif
